package dbquery

import (
	"fmt"
	"git.sr.ht/~mil/mobroute/util/utilfuncs"
	"github.com/jmoiron/sqlx"
	"strconv"
	"strings"
)

// QueryBoundsMatch is a querying function that determines if the passed
// param matches for the feed_id for the DB conneciton min/max lat/lon
func QueryBoundsMatch(
	DB *sqlx.DB,
	queryFeedIDs []int,
	coord [2]float64,
) error {
	var (
		// TODO: Should this just be a call to mobsql rather then a direct query?
		query = fmt.Sprintf(
			`
				select count(*) from (
					select 
						(? between min_lat and max_lat) and
						(? between min_lon and max_lon) as matches_bounds
					from _vfeedsmetadata
					where feed_id in (%s) and matches_bounds
				) g
			`,
			strings.Join(utilfuncs.ArrayWithValue(len(queryFeedIDs), "?"), ","),
		)
		params = append(
			[]any{coord[0], coord[1]},
			utilfuncs.Anyify(utilfuncs.ArrayIntToArryString(queryFeedIDs))...,
		)
	)

	if value, err := QueryValue(DB, query, params); err != nil {
		return err
	} else if value == "" {
		return fmt.Errorf("%v doesn't match any of the provided bounds", coord)
	} else if matches, err := strconv.Atoi(value); err != nil {
		return err
	} else if matches < 1 {
		return fmt.Errorf("%v doesn't match any of the provided bounds - matches: %d, check bounds for feed ids : %v", coord, matches, queryFeedIDs)
	} else {
		return nil
	}
}
